@@ -557,6 +557,35 @@ DWORD WINAPI GetThreadId(HANDLE Thread)
     return HandleToULong(tbi.ClientId.UniqueThread);
 }
 
+/**********************************************************************
+ * GetProcessIdOfThread [KERNEL32.@]
+ *
+ * Retrieve process identifier given thread belongs to.
+ *
+ * PARAMS
+ *  Thread [I] The thread identifier.
+ *
+ * RETURNS
+ *    Success: Process identifier
+ *    Failure: 0
+ */
+DWORD WINAPI GetProcessIdOfThread(HANDLE Thread)
+{
+    THREAD_BASIC_INFORMATION tbi;
+    NTSTATUS status;
+
+    TRACE("(%p)\n", Thread);
+
+    status = NtQueryInformationThread(Thread, ThreadBasicInformation, &tbi,
+                                      sizeof(tbi), NULL);
+    if (status)
+    {
+        SetLastError( RtlNtStatusToDosError(status) );
+        return 0;
+    }
+
+    return HandleToULong(tbi.ClientId.UniqueProcess);
+}
 
 /***********************************************************************
  * GetCurrentThread [KERNEL32.@]  Gets pseudohandle for current thread
@@ -607,7 +636,7 @@ __ASM_STDCALL_FUNC( GetCurrentThreadId, 0, ".byte 0x64\n\tmovl 0x24,%eax\n\tret"
 /* HANDLE WINAPI GetProcessHeap(void) */
 __ASM_STDCALL_FUNC( GetProcessHeap, 0, ".byte 0x64\n\tmovl 0x30,%eax\n\tmovl 0x18(%eax),%eax\n\tret");
 
-#elif defined(__x86_64__)
+#elif defined(__x86_64__) && !defined(__APPLE__)
 
 /***********************************************************************
  *		SetLastError (KERNEL32.@)
