@@ -33,6 +33,7 @@
 #include "winternl.h"
 #include "excpt.h"
 #include "winioctl.h"
+#include "ddk/csq.h"
 #include "ddk/ntddk.h"
 #include "ddk/ntifs.h"
 #include "wine/unicode.h"
@@ -63,14 +64,6 @@ KSERVICE_TABLE_DESCRIPTOR KeServiceDescriptorTable[4] = { { 0 } };
 typedef void (WINAPI *PCREATE_PROCESS_NOTIFY_ROUTINE)(HANDLE,HANDLE,BOOLEAN);
 typedef void (WINAPI *PCREATE_THREAD_NOTIFY_ROUTINE)(HANDLE,HANDLE,BOOLEAN);
 
-static struct list Irps = LIST_INIT(Irps);
-
-struct IrpInstance
-{
-    struct list entry;
-    IRP *irp;
-};
-
 /* tid of the thread running client request */
 static DWORD request_thread;
 
@@ -135,76 +128,164 @@ static HANDLE get_device_manager(void)
     return ret;
 }
 
-/* process an ioctl request for a given device */
-static NTSTATUS process_ioctl( DEVICE_OBJECT *device, ULONG code, void *in_buff, ULONG in_size,
-                               void *out_buff, ULONG *out_size )
+static NTSTATUS dispatch_irp( DEVICE_OBJECT *device, IRP *irp )
 {
-    IRP irp;
-    MDL mdl;
-    IO_STACK_LOCATION irpsp;
-    PDRIVER_DISPATCH dispatch = device->DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL];
-    NTSTATUS status;
     LARGE_INTEGER count;
+    FILE_OBJECT file;
 
-    TRACE( "ioctl %x device %p in_size %u out_size %u\n", code, device, in_size, *out_size );
+    irp->RequestorMode = UserMode;
+    irp->Tail.Overlay.OriginalFileObject = &file;
 
-    /* so we can spot things that we should initialize */
-    memset( &irp, 0x55, sizeof(irp) );
-    memset( &irpsp, 0x66, sizeof(irpsp) );
-    memset( &mdl, 0x77, sizeof(mdl) );
-
-    irp.RequestorMode = UserMode;
-    if ((code & 3) == METHOD_BUFFERED)
-    {
-        irp.AssociatedIrp.SystemBuffer = HeapAlloc( GetProcessHeap(), 0, max( in_size, *out_size ) );
-        if (!irp.AssociatedIrp.SystemBuffer)
-            return STATUS_NO_MEMORY;
-        memcpy( irp.AssociatedIrp.SystemBuffer, in_buff, in_size );
-    }
-    else
-        irp.AssociatedIrp.SystemBuffer = in_buff;
-    irp.UserBuffer = out_buff;
-    irp.MdlAddress = &mdl;
-    irp.Tail.Overlay.s.u2.CurrentStackLocation = &irpsp;
-    irp.UserIosb = NULL;
-
-    irpsp.MajorFunction = IRP_MJ_DEVICE_CONTROL;
-    irpsp.Parameters.DeviceIoControl.OutputBufferLength = *out_size;
-    irpsp.Parameters.DeviceIoControl.InputBufferLength = in_size;
-    irpsp.Parameters.DeviceIoControl.IoControlCode = code;
-    irpsp.Parameters.DeviceIoControl.Type3InputBuffer = in_buff;
-    irpsp.DeviceObject = device;
-    irpsp.CompletionRoutine = NULL;
-
-    mdl.Next = NULL;
-    mdl.Size = 0;
-    mdl.StartVa = out_buff;
-    mdl.ByteCount = *out_size;
-    mdl.ByteOffset = 0;
-
-    device->CurrentIrp = &irp;
+    memset( &file, 0x88, sizeof(file) );
+    file.FsContext = NULL;
+    file.FsContext2 = NULL;
 
     KeQueryTickCount( &count );  /* update the global KeTickCount */
 
-    if (TRACE_ON(relay))
-        DPRINTF( "%04x:Call driver dispatch %p (device=%p,irp=%p)\n",
-                 GetCurrentThreadId(), dispatch, device, &irp );
+    device->CurrentIrp = irp;
 
-    status = dispatch( device, &irp );
+    IoCallDriver( device, irp );
 
-    if (TRACE_ON(relay))
-        DPRINTF( "%04x:Ret  driver dispatch %p (device=%p,irp=%p) retval=%08x\n",
-                 GetCurrentThreadId(), dispatch, device, &irp, status );
+    device->CurrentIrp = NULL;
 
-    *out_size = (irp.IoStatus.u.Status >= 0) ? irp.IoStatus.Information : 0;
-    if ((code & 3) == METHOD_BUFFERED)
-    {
-        if (out_buff) memcpy( out_buff, irp.AssociatedIrp.SystemBuffer, *out_size );
-        HeapFree( GetProcessHeap(), 0, irp.AssociatedIrp.SystemBuffer );
-    }
-    return irp.IoStatus.u.Status;
+    return STATUS_SUCCESS;
 }
 
+/* process a read request for a given device */
+static NTSTATUS dispatch_read( DEVICE_OBJECT *device, const irp_params_t *params,
+                               void *in_buff, ULONG in_size, ULONG out_size, HANDLE irp_handle )
+{
+    IRP *irp;
+    void *out_buff;
+    LARGE_INTEGER offset;
+    IO_STACK_LOCATION *irpsp;
+
+    TRACE( "device %p size %u\n", device, out_size );
+
+    if (!(out_buff = HeapAlloc( GetProcessHeap(), 0, out_size ))) return STATUS_NO_MEMORY;
+
+    offset.QuadPart = params->read.pos;
+
+    /* note: we abuse UserIosb to store the server irp handle */
+    if (!(irp = IoBuildSynchronousFsdRequest( IRP_MJ_READ, device, out_buff, out_size,
+                                              &offset, NULL, irp_handle )))
+    {
+        HeapFree( GetProcessHeap(), 0, out_buff );
+        return STATUS_NO_MEMORY;
+    }
+
+    irpsp = IoGetNextIrpStackLocation( irp );
+    irpsp->Parameters.Read.Key = params->read.key;
+
+    return dispatch_irp( device, irp );
+}
+
+/* process a write request for a given device */
+static NTSTATUS dispatch_write( DEVICE_OBJECT *device, const irp_params_t *params,
+                                void *in_buff, ULONG in_size, ULONG out_size, HANDLE irp_handle )
+{
+    IRP *irp;
+    LARGE_INTEGER offset;
+    IO_STACK_LOCATION *irpsp;
+
+    TRACE( "device %p size %u\n", device, in_size );
+
+    offset.QuadPart = params->write.pos;
+
+    /* note: we abuse UserIosb to store the server irp handle */
+    if (!(irp = IoBuildSynchronousFsdRequest( IRP_MJ_WRITE, device, in_buff, in_size,
+                                              &offset, NULL, irp_handle )))
+        return STATUS_NO_MEMORY;
+
+    irpsp = IoGetNextIrpStackLocation( irp );
+    irpsp->Parameters.Write.Key = params->write.key;
+
+    return dispatch_irp( device, irp );
+}
+
+/* process a flush request for a given device */
+static NTSTATUS dispatch_flush( DEVICE_OBJECT *device, const irp_params_t *params,
+                                void *in_buff, ULONG in_size, ULONG out_size, HANDLE irp_handle )
+{
+    IRP *irp;
+
+    TRACE( "device %p\n", device );
+
+    /* note: we abuse UserIosb to store the server irp handle */
+    if (!(irp = IoBuildSynchronousFsdRequest( IRP_MJ_FLUSH_BUFFERS, device, in_buff, in_size,
+                                              NULL, NULL, irp_handle )))
+        return STATUS_NO_MEMORY;
+
+    return dispatch_irp( device, irp );
+}
+
+/* process an ioctl request for a given device */
+static NTSTATUS dispatch_ioctl( DEVICE_OBJECT *device, const irp_params_t *params,
+                                void *in_buff, ULONG in_size, ULONG out_size, HANDLE irp_handle )
+{
+    IRP *irp;
+    void *out_buff = NULL;
+
+    TRACE( "ioctl %x device %p in_size %u out_size %u\n", params->ioctl.code, device, in_size, out_size );
+
+    if ((params->ioctl.code & 3) == METHOD_BUFFERED) out_size = max( in_size, out_size );
+
+    if (out_size)
+    {
+        if (!(out_buff = HeapAlloc( GetProcessHeap(), 0, out_size ))) return STATUS_NO_MEMORY;
+        if ((params->ioctl.code & 3) == METHOD_BUFFERED)
+        {
+            memcpy( out_buff, in_buff, in_size );
+            in_buff = out_buff;
+        }
+    }
+
+    /* note: we abuse UserIosb to store the server handle to the ioctl */
+    irp = IoBuildDeviceIoControlRequest( params->ioctl.code, device, in_buff, in_size, out_buff, out_size,
+                                         FALSE, NULL, irp_handle );
+    if (!irp)
+    {
+        HeapFree( GetProcessHeap(), 0, out_buff );
+        return STATUS_NO_MEMORY;
+    }
+    return dispatch_irp( device, irp );
+}
+
+typedef NTSTATUS (*dispatch_func)( DEVICE_OBJECT *device, const irp_params_t *params,
+                                   void *in_buff, ULONG in_size, ULONG out_size, HANDLE irp_handle );
+
+static const dispatch_func dispatch_funcs[IRP_MJ_MAXIMUM_FUNCTION + 1] =
+{
+    NULL,              /* IRP_MJ_CREATE */
+    NULL,              /* IRP_MJ_CREATE_NAMED_PIPE */
+    NULL,              /* IRP_MJ_CLOSE */
+    dispatch_read,     /* IRP_MJ_READ */
+    dispatch_write,    /* IRP_MJ_WRITE */
+    NULL,              /* IRP_MJ_QUERY_INFORMATION */
+    NULL,              /* IRP_MJ_SET_INFORMATION */
+    NULL,              /* IRP_MJ_QUERY_EA */
+    NULL,              /* IRP_MJ_SET_EA */
+    dispatch_flush,    /* IRP_MJ_FLUSH_BUFFERS */
+    NULL,              /* IRP_MJ_QUERY_VOLUME_INFORMATION */
+    NULL,              /* IRP_MJ_SET_VOLUME_INFORMATION */
+    NULL,              /* IRP_MJ_DIRECTORY_CONTROL */
+    NULL,              /* IRP_MJ_FILE_SYSTEM_CONTROL */
+    dispatch_ioctl,    /* IRP_MJ_DEVICE_CONTROL */
+    NULL,              /* IRP_MJ_INTERNAL_DEVICE_CONTROL */
+    NULL,              /* IRP_MJ_SHUTDOWN */
+    NULL,              /* IRP_MJ_LOCK_CONTROL */
+    NULL,              /* IRP_MJ_CLEANUP */
+    NULL,              /* IRP_MJ_CREATE_MAILSLOT */
+    NULL,              /* IRP_MJ_QUERY_SECURITY */
+    NULL,              /* IRP_MJ_SET_SECURITY */
+    NULL,              /* IRP_MJ_POWER */
+    NULL,              /* IRP_MJ_SYSTEM_CONTROL */
+    NULL,              /* IRP_MJ_DEVICE_CHANGE */
+    NULL,              /* IRP_MJ_QUERY_QUOTA */
+    NULL,              /* IRP_MJ_SET_QUOTA */
+    NULL,              /* IRP_MJ_PNP */
+};
+
 
 /***********************************************************************
  *           wine_ntoskrnl_main_loop   (Not a Windows API)
@@ -212,10 +293,10 @@ static NTSTATUS process_ioctl( DEVICE_OBJECT *device, ULONG code, void *in_buff,
 NTSTATUS CDECL wine_ntoskrnl_main_loop( HANDLE stop_event )
 {
     HANDLE manager = get_device_manager();
-    obj_handle_t ioctl = 0;
+    HANDLE irp = 0;
     NTSTATUS status = STATUS_SUCCESS;
-    ULONG code = 0;
-    void *in_buff, *out_buff = NULL;
+    irp_params_t irp_params;
+    void *in_buff;
     DEVICE_OBJECT *device = NULL;
     ULONG in_size = 4096, out_size = 0;
     HANDLE handles[2];
@@ -236,15 +317,14 @@ NTSTATUS CDECL wine_ntoskrnl_main_loop( HANDLE stop_event )
         SERVER_START_REQ( get_next_device_request )
         {
             req->manager = wine_server_obj_handle( manager );
-            req->prev = ioctl;
+            req->prev = wine_server_obj_handle( irp );
             req->status = status;
-            wine_server_add_data( req, out_buff, out_size );
             wine_server_set_reply( req, in_buff, in_size );
             if (!(status = wine_server_call( req )))
             {
-                code       = reply->code;
-                ioctl      = reply->next;
+                irp        = wine_server_ptr_handle( reply->next );
                 device     = wine_server_get_ptr( reply->user_ptr );
+                irp_params = reply->params;
                 client_tid = reply->client_tid;
                 client_pid = reply->client_pid;
                 in_size    = reply->in_size;
@@ -252,7 +332,7 @@ NTSTATUS CDECL wine_ntoskrnl_main_loop( HANDLE stop_event )
             }
             else
             {
-                ioctl = 0; /* no previous ioctl */
+                irp = 0; /* no previous irp */
                 out_size = 0;
                 in_size = reply->in_size;
             }
@@ -262,10 +342,17 @@ NTSTATUS CDECL wine_ntoskrnl_main_loop( HANDLE stop_event )
         switch(status)
         {
         case STATUS_SUCCESS:
-            HeapFree( GetProcessHeap(), 0, out_buff );
-            if (out_size) out_buff = HeapAlloc( GetProcessHeap(), 0, out_size );
-            else out_buff = NULL;
-            status = process_ioctl( device, code, in_buff, in_size, out_buff, &out_size );
+            if (irp_params.major > IRP_MJ_MAXIMUM_FUNCTION ||
+                !dispatch_funcs[irp_params.major] ||
+                !device->DriverObject->MajorFunction[irp_params.major])
+            {
+                WARN( "unsupported request %u\n", irp_params.major );
+                status = STATUS_NOT_SUPPORTED;
+                break;
+            }
+            status = dispatch_funcs[irp_params.major]( device, &irp_params,
+                                                       in_buff, in_size, out_size, irp );
+            if (status == STATUS_SUCCESS) irp = 0;  /* status reported by IoCompleteRequest */
             break;
         case STATUS_BUFFER_OVERFLOW:
             HeapFree( GetProcessHeap(), 0, in_buff );
@@ -273,11 +360,15 @@ NTSTATUS CDECL wine_ntoskrnl_main_loop( HANDLE stop_event )
             /* restart with larger buffer */
             break;
         case STATUS_PENDING:
-            if (WaitForMultipleObjects( 2, handles, FALSE, INFINITE ) == WAIT_OBJECT_0)
+            for (;;)
             {
-                HeapFree( GetProcessHeap(), 0, in_buff );
-                HeapFree( GetProcessHeap(), 0, out_buff );
-                return STATUS_SUCCESS;
+                DWORD ret = WaitForMultipleObjectsEx( 2, handles, FALSE, INFINITE, TRUE );
+                if (ret == WAIT_OBJECT_0)
+                {
+                    HeapFree( GetProcessHeap(), 0, in_buff );
+                    return STATUS_SUCCESS;
+                }
+                if (ret != WAIT_IO_COMPLETION) break;
             }
             break;
         }
@@ -395,8 +486,18 @@ PIRP WINAPI IoAllocateIrp( CCHAR stack_size, BOOLEAN charge_quota )
  */
 void WINAPI IoFreeIrp( IRP *irp )
 {
+    MDL *mdl;
+
     TRACE( "%p\n", irp );
 
+    mdl = irp->MdlAddress;
+    while (mdl)
+    {
+        MDL *next = mdl->Next;
+        IoFreeMdl( mdl );
+        mdl = next;
+    }
+
     ExFreePool( irp );
 }
 
@@ -414,24 +515,21 @@ PVOID WINAPI IoAllocateErrorLogEntry( PVOID IoObject, UCHAR EntrySize )
 /***********************************************************************
  *           IoAllocateMdl  (NTOSKRNL.EXE.@)
  */
-PMDL WINAPI IoAllocateMdl( PVOID VirtualAddress, ULONG Length, BOOLEAN SecondaryBuffer, BOOLEAN ChargeQuota, PIRP Irp )
+PMDL WINAPI IoAllocateMdl( PVOID va, ULONG length, BOOLEAN secondary, BOOLEAN charge_quota, IRP *irp )
 {
     PMDL mdl;
-    ULONG_PTR address = (ULONG_PTR)VirtualAddress;
+    ULONG_PTR address = (ULONG_PTR)va;
     ULONG_PTR page_address;
     SIZE_T nb_pages, mdl_size;
 
-    TRACE("(%p, %u, %i, %i, %p)\n", VirtualAddress, Length, SecondaryBuffer, ChargeQuota, Irp);
+    TRACE("(%p, %u, %i, %i, %p)\n", va, length, secondary, charge_quota, irp);
 
-    if (Irp)
-        FIXME("Attaching the MDL to an IRP is not yet supported\n");
-
-    if (ChargeQuota)
+    if (charge_quota)
         FIXME("Charge quota is not yet supported\n");
 
     /* FIXME: We suppose that page size is 4096 */
     page_address = address & ~(4096 - 1);
-    nb_pages = (((address + Length - 1) & ~(4096 - 1)) - page_address) / 4096 + 1;
+    nb_pages = (((address + length - 1) & ~(4096 - 1)) - page_address) / 4096 + 1;
 
     mdl_size = sizeof(MDL) + nb_pages * sizeof(PVOID);
 
@@ -440,11 +538,24 @@ PMDL WINAPI IoAllocateMdl( PVOID VirtualAddress, ULONG Length, BOOLEAN Secondary
         return NULL;
 
     mdl->Size = mdl_size;
-    mdl->Process = IoGetCurrentProcess();
+    mdl->Process = NULL; /* FIXME: IoGetCurrentProcess */
     mdl->StartVa = (PVOID)page_address;
-    mdl->ByteCount = Length;
+    mdl->ByteCount = length;
     mdl->ByteOffset = address - page_address;
 
+    if (!irp) return mdl;
+
+    if (secondary)  /* add it at the end */
+    {
+        MDL **pmdl = &irp->MdlAddress;
+        while (*pmdl) pmdl = &(*pmdl)->Next;
+        *pmdl = mdl;
+    }
+    else
+    {
+        mdl->Next = irp->MdlAddress;
+        irp->MdlAddress = mdl;
+    }
     return mdl;
 }
 
@@ -452,10 +563,9 @@ PMDL WINAPI IoAllocateMdl( PVOID VirtualAddress, ULONG Length, BOOLEAN Secondary
 /***********************************************************************
  *           IoFreeMdl  (NTOSKRNL.EXE.@)
  */
-VOID WINAPI IoFreeMdl(PMDL mdl)
+void WINAPI IoFreeMdl(PMDL mdl)
 {
-    FIXME("partial stub: %p\n", mdl);
-
+    TRACE("%p\n", mdl);
     HeapFree(GetProcessHeap(), 0, mdl);
 }
 
@@ -486,48 +596,93 @@ PDEVICE_OBJECT WINAPI IoAttachDeviceToDeviceStack( DEVICE_OBJECT *source,
 /***********************************************************************
  *           IoBuildDeviceIoControlRequest  (NTOSKRNL.EXE.@)
  */
-PIRP WINAPI IoBuildDeviceIoControlRequest( ULONG IoControlCode,
-                                           PDEVICE_OBJECT DeviceObject,
-                                           PVOID InputBuffer,
-                                           ULONG InputBufferLength,
-                                           PVOID OutputBuffer,
-                                           ULONG OutputBufferLength,
-                                           BOOLEAN InternalDeviceIoControl,
-                                           PKEVENT Event,
-                                           PIO_STATUS_BLOCK IoStatusBlock )
+PIRP WINAPI IoBuildDeviceIoControlRequest( ULONG code, PDEVICE_OBJECT device,
+                                           PVOID in_buff, ULONG in_len,
+                                           PVOID out_buff, ULONG out_len,
+                                           BOOLEAN internal, PKEVENT event,
+                                           PIO_STATUS_BLOCK iosb )
 {
     PIRP irp;
     PIO_STACK_LOCATION irpsp;
-    struct IrpInstance *instance;
 
     TRACE( "%x, %p, %p, %u, %p, %u, %u, %p, %p\n",
-           IoControlCode, DeviceObject, InputBuffer, InputBufferLength,
-           OutputBuffer, OutputBufferLength, InternalDeviceIoControl,
-           Event, IoStatusBlock );
+           code, device, in_buff, in_len, out_buff, out_len, internal, event, iosb );
 
-    if (DeviceObject == NULL)
+    if (device == NULL)
         return NULL;
 
-    irp = IoAllocateIrp( DeviceObject->StackSize, FALSE );
+    irp = IoAllocateIrp( device->StackSize, FALSE );
     if (irp == NULL)
         return NULL;
 
-    instance = HeapAlloc( GetProcessHeap(), 0, sizeof(struct IrpInstance) );
-    if (instance == NULL)
+    irpsp = IoGetNextIrpStackLocation( irp );
+    irpsp->MajorFunction = internal ? IRP_MJ_INTERNAL_DEVICE_CONTROL : IRP_MJ_DEVICE_CONTROL;
+    irpsp->Parameters.DeviceIoControl.IoControlCode = code;
+    irpsp->Parameters.DeviceIoControl.InputBufferLength = in_len;
+    irpsp->Parameters.DeviceIoControl.OutputBufferLength = out_len;
+    irpsp->DeviceObject = device;
+    irpsp->CompletionRoutine = NULL;
+
+    switch (code & 3)
     {
-        IoFreeIrp( irp );
-        return NULL;
+    case METHOD_BUFFERED:
+        irp->AssociatedIrp.SystemBuffer = in_buff;
+        break;
+    case METHOD_IN_DIRECT:
+    case METHOD_OUT_DIRECT:
+        irp->AssociatedIrp.SystemBuffer = in_buff;
+        IoAllocateMdl( out_buff, out_len, FALSE, FALSE, irp );
+        break;
+    case METHOD_NEITHER:
+        irpsp->Parameters.DeviceIoControl.Type3InputBuffer = in_buff;
+        break;
     }
-    instance->irp = irp;
-    list_add_tail( &Irps, &instance->entry );
+
+    irp->RequestorMode = KernelMode;
+    irp->UserBuffer = out_buff;
+    irp->UserIosb = iosb;
+    irp->UserEvent = event;
+    return irp;
+}
+
+
+/**********************************************************
+ *           IoBuildSynchronousFsdRequest  (NTOSKRNL.EXE.@)
+ */
+PIRP WINAPI IoBuildSynchronousFsdRequest(ULONG majorfunc, PDEVICE_OBJECT device,
+                                         PVOID buffer, ULONG length, PLARGE_INTEGER startoffset,
+                                         PKEVENT event, PIO_STATUS_BLOCK iosb)
+{
+    PIRP irp;
+    PIO_STACK_LOCATION irpsp;
+
+    TRACE("(%d %p %p %d %p %p %p)\n", majorfunc, device, buffer, length, startoffset, event, iosb);
+
+    if (!(irp = IoAllocateIrp( device->StackSize, FALSE ))) return NULL;
 
     irpsp = IoGetNextIrpStackLocation( irp );
-    irpsp->MajorFunction = InternalDeviceIoControl ?
-            IRP_MJ_INTERNAL_DEVICE_CONTROL : IRP_MJ_DEVICE_CONTROL;
-    irpsp->Parameters.DeviceIoControl.IoControlCode = IoControlCode;
-    irp->UserIosb = IoStatusBlock;
-    irp->UserEvent = Event;
+    irpsp->MajorFunction = majorfunc;
+    irpsp->DeviceObject = device;
+    irpsp->CompletionRoutine = NULL;
 
+    irp->AssociatedIrp.SystemBuffer = buffer;
+    if (device->Flags & DO_DIRECT_IO) IoAllocateMdl( buffer, length, FALSE, FALSE, irp );
+
+    switch (majorfunc)
+    {
+    case IRP_MJ_READ:
+        irpsp->Parameters.Read.Length = length;
+        irpsp->Parameters.Read.ByteOffset = *startoffset;
+        break;
+    case IRP_MJ_WRITE:
+        irpsp->Parameters.Write.Length = length;
+        irpsp->Parameters.Write.ByteOffset = *startoffset;
+        break;
+    }
+    irp->RequestorMode = KernelMode;
+    irp->UserIosb = iosb;
+    irp->UserEvent = event;
+    irp->UserBuffer = buffer;
     return irp;
 }
 
@@ -731,9 +886,21 @@ NTSTATUS  WINAPI IoGetDeviceObjectPointer( UNICODE_STRING *name, ACCESS_MASK acc
 {
     FIXME( "stub: %s %x %p %p\n", debugstr_us(name), access, file, device );
 
-    return STATUS_NOT_IMPLEMENTED;
+    *file  = NULL;
+    *device  = NULL;
+
+    return STATUS_SUCCESS;
 }
 
+/***********************************************************************
+ *           IoGetAttachedDevice   (NTOSKRNL.EXE.@)
+ */
+PDEVICE_OBJECT WINAPI IoGetAttachedDevice( PDEVICE_OBJECT device )
+{
+    FIXME( "stub: %p\n", device );
+
+    return device;
+}
 
 /***********************************************************************
  *           IoGetDeviceProperty   (NTOSKRNL.EXE.@)
@@ -756,13 +923,20 @@ NTSTATUS WINAPI IoCallDriver( DEVICE_OBJECT *device, IRP *irp )
     IO_STACK_LOCATION *irpsp;
     NTSTATUS status;
 
-    TRACE( "%p %p\n", device, irp );
-
     --irp->CurrentLocation;
     irpsp = --irp->Tail.Overlay.s.u2.CurrentStackLocation;
     dispatch = device->DriverObject->MajorFunction[irpsp->MajorFunction];
+
+    if (TRACE_ON(relay))
+        DPRINTF( "%04x:Call driver dispatch %p (device=%p,irp=%p)\n",
+                 GetCurrentThreadId(), dispatch, device, irp );
+
     status = dispatch( device, irp );
 
+    if (TRACE_ON(relay))
+        DPRINTF( "%04x:Ret  driver dispatch %p (device=%p,irp=%p) retval=%08x\n",
+                 GetCurrentThreadId(), dispatch, device, irp, status );
+
     return status;
 }
 
@@ -930,14 +1104,12 @@ VOID WINAPI IoCompleteRequest( IRP *irp, UCHAR priority_boost )
 {
     IO_STACK_LOCATION *irpsp;
     PIO_COMPLETION_ROUTINE routine;
-    IO_STATUS_BLOCK *iosb;
-    struct IrpInstance *instance;
     NTSTATUS status, stat;
+    HANDLE handle;
     int call_flag = 0;
 
     TRACE( "%p %u\n", irp, priority_boost );
 
-    iosb = irp->UserIosb;
     status = irp->IoStatus.u.Status;
     while (irp->CurrentLocation <= irp->StackCount)
     {
@@ -964,21 +1136,30 @@ VOID WINAPI IoCompleteRequest( IRP *irp, UCHAR priority_boost )
                 return;
         }
     }
-    if (iosb && STATUS_SUCCESS == status)
+
+    handle = (HANDLE)irp->UserIosb;
+    if (handle)
     {
-        iosb->u.Status = irp->IoStatus.u.Status;
-        iosb->Information = irp->IoStatus.Information;
-    }
-    LIST_FOR_EACH_ENTRY( instance, &Irps, struct IrpInstance, entry )
-    {
-        if (instance->irp == irp)
+        HANDLE manager = get_device_manager();
+        void *out_buff = irp->UserBuffer;
+
+        SERVER_START_REQ( set_irp_result )
         {
-            list_remove( &instance->entry );
-            HeapFree( GetProcessHeap(), 0, instance );
-            IoFreeIrp( irp );
-            break;
+            req->manager = wine_server_obj_handle( manager );
+            req->handle  = wine_server_obj_handle( handle );
+            req->status  = irp->IoStatus.u.Status;
+            if (irp->IoStatus.u.Status >= 0)
+            {
+                req->size = irp->IoStatus.Information;
+                if (out_buff) wine_server_add_data( req, out_buff, irp->IoStatus.Information );
+            }
+            wine_server_call( req );
         }
+        SERVER_END_REQ;
+        HeapFree( GetProcessHeap(), 0, out_buff );
     }
+
+    IoFreeIrp( irp );
 }
 
 
@@ -1402,6 +1583,13 @@ KPRIORITY WINAPI KeSetPriorityThread( PKTHREAD Thread, KPRIORITY Priority )
     return Priority;
 }
 
+/***********************************************************************
+ *           KeSetSystemAffinityThread   (NTOSKRNL.EXE.@)
+ */
+VOID WINAPI KeSetSystemAffinityThread(KAFFINITY Affinity)
+{
+    FIXME("(%lx) stub\n", Affinity);
+}
 
 /***********************************************************************
  *           KeWaitForSingleObject   (NTOSKRNL.EXE.@)
@@ -1830,7 +2018,8 @@ NTSTATUS WINAPI PsSetLoadImageNotifyRoutine(PLOAD_IMAGE_NOTIFY_ROUTINE routine)
  */
 NTSTATUS WINAPI PsLookupProcessByProcessId(HANDLE processid, PEPROCESS *process)
 {
-    FIXME("(%p %p) stub\n", processid, process);
+    static int once;
+    if (!once++) FIXME("(%p %p) stub\n", processid, process);
     return STATUS_NOT_IMPLEMENTED;
 }
 
@@ -1927,3 +2116,58 @@ VOID WINAPI IoStartNextPacket(PDEVICE_OBJECT deviceobject, BOOLEAN cancelable)
 {
     FIXME("(%p %d) stub\n", deviceobject, cancelable);
 }
+
+/*****************************************************
+ *           ObQueryNameString  (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI ObQueryNameString(PVOID object, POBJECT_NAME_INFORMATION name, ULONG maxlength, PULONG returnlength)
+{
+    FIXME("(%p %p %u %p) stub\n", object, name, maxlength, returnlength);
+    return STATUS_NOT_IMPLEMENTED;
+}
+
+/*****************************************************
+ *           IoRegisterPlugPlayNotification  (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI IoRegisterPlugPlayNotification(IO_NOTIFICATION_EVENT_CATEGORY category, ULONG flags, PVOID data,
+                                               PDRIVER_OBJECT driver, PDRIVER_NOTIFICATION_CALLBACK_ROUTINE callback,
+                                               PVOID context, PVOID *notification)
+{
+    FIXME("(%u %u %p %p %p %p %p) stub\n", category, flags, data, driver, callback, context, notification);
+    return STATUS_SUCCESS;
+}
+
+/*****************************************************
+ *           IoCsqInitialize  (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI IoCsqInitialize(PIO_CSQ csq, PIO_CSQ_INSERT_IRP insert_irp, PIO_CSQ_REMOVE_IRP remove_irp,
+                                PIO_CSQ_PEEK_NEXT_IRP peek_irp, PIO_CSQ_ACQUIRE_LOCK acquire_lock,
+                                PIO_CSQ_RELEASE_LOCK release_lock, PIO_CSQ_COMPLETE_CANCELED_IRP complete_irp)
+{
+    FIXME("(%p %p %p %p %p %p %p) stub\n",
+          csq, insert_irp, remove_irp, peek_irp, acquire_lock, release_lock, complete_irp);
+    return STATUS_SUCCESS;
+}
+
+/*****************************************************
+ *           ExInterlockedRemoveHeadList  (NTOSKRNL.EXE.@)
+ */
+PLIST_ENTRY WINAPI ExInterlockedRemoveHeadList(PLIST_ENTRY head, PKSPIN_LOCK lock)
+{
+    FIXME("(%p %p) stub\n", head, lock);
+    return NULL;
+}
+
+/***********************************************************************
+ *           ExfInterlockedRemoveHeadList   (NTOSKRNL.EXE.@)
+ */
+#ifdef DEFINE_FASTCALL2_ENTRYPOINT
+DEFINE_FASTCALL2_ENTRYPOINT( ExfInterlockedRemoveHeadList )
+PLIST_ENTRY WINAPI __regs_ExfInterlockedRemoveHeadList(PLIST_ENTRY head, PKSPIN_LOCK lock)
+#else
+PLIST_ENTRY WINAPI ExfInterlockedRemoveHeadList(PLIST_ENTRY head, PKSPIN_LOCK lock)
+#endif
+{
+    FIXME("(%p %p) stub\n", head, lock);
+    return ExInterlockedRemoveHeadList( head, lock );
+}
