@@ -22,53 +22,217 @@
 
 #include "windef.h"
 #include "winbase.h"
-
 #include "qmgr.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(qmgr);
 
+struct copy_error
+{
+    IBackgroundCopyError  IBackgroundCopyError_iface;
+    LONG                  refs;
+    BG_ERROR_CONTEXT      context;
+    HRESULT               code;
+    IBackgroundCopyFile2 *file;
+};
+
+static inline struct copy_error *impl_from_IBackgroundCopyError(IBackgroundCopyError *iface)
+{
+    return CONTAINING_RECORD(iface, struct copy_error, IBackgroundCopyError_iface);
+}
+
+static HRESULT WINAPI copy_error_QueryInterface(
+    IBackgroundCopyError *iface,
+    REFIID riid,
+    void **obj)
+{
+    struct copy_error *error = impl_from_IBackgroundCopyError(iface);
+
+    TRACE("(%p)->(%s %p)\n", error, debugstr_guid(riid), obj);
+
+    if (IsEqualGUID(riid, &IID_IUnknown) || IsEqualGUID(riid, &IID_IBackgroundCopyError))
+    {
+        *obj = &error->IBackgroundCopyError_iface;
+    }
+    else
+    {
+        *obj = NULL;
+        WARN("interface %s not supported\n", debugstr_guid(riid));
+        return E_NOINTERFACE;
+    }
+
+    IBackgroundCopyError_AddRef(iface);
+    return S_OK;
+}
+
+static ULONG WINAPI copy_error_AddRef(
+    IBackgroundCopyError *iface)
+{
+    struct copy_error *error = impl_from_IBackgroundCopyError(iface);
+    LONG refs = InterlockedIncrement(&error->refs);
+    TRACE("(%p)->(%d)\n", error, refs);
+    return refs;
+}
+
+static ULONG WINAPI copy_error_Release(
+    IBackgroundCopyError *iface)
+{
+    struct copy_error *error = impl_from_IBackgroundCopyError(iface);
+    LONG refs = InterlockedDecrement(&error->refs);
+
+    TRACE("(%p)->(%d)\n", error, refs);
+
+    if (!refs)
+    {
+        if (error->file) IBackgroundCopyFile2_Release(error->file);
+        HeapFree(GetProcessHeap(), 0, error);
+    }
+    return refs;
+}
+
+static HRESULT WINAPI copy_error_GetError(
+    IBackgroundCopyError *iface,
+    BG_ERROR_CONTEXT *pContext,
+    HRESULT *pCode)
+{
+    struct copy_error *error = impl_from_IBackgroundCopyError(iface);
+
+    TRACE("(%p)->(%p %p)\n", error, pContext, pCode);
+
+    *pContext = error->context;
+    *pCode = error->code;
+
+    TRACE("returning context %u error code 0x%08x\n", error->context, error->code);
+    return S_OK;
+}
+
+static HRESULT WINAPI copy_error_GetFile(
+    IBackgroundCopyError *iface,
+    IBackgroundCopyFile **pVal)
+{
+    struct copy_error *error = impl_from_IBackgroundCopyError(iface);
+
+    TRACE("(%p)->(%p)\n", error, pVal);
+
+    if (error->file)
+    {
+        IBackgroundCopyFile2_AddRef(error->file);
+        *pVal = (IBackgroundCopyFile *)error->file;
+        return S_OK;
+    }
+    *pVal = NULL;
+    return BG_E_FILE_NOT_AVAILABLE;
+}
+
+static HRESULT WINAPI copy_error_GetErrorDescription(
+    IBackgroundCopyError *iface,
+    DWORD LanguageId,
+    LPWSTR *pErrorDescription)
+{
+    struct copy_error *error = impl_from_IBackgroundCopyError(iface);
+    FIXME("(%p)->(%p)\n", error, pErrorDescription);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI copy_error_GetErrorContextDescription(
+    IBackgroundCopyError *iface,
+    DWORD LanguageId,
+    LPWSTR *pContextDescription)
+{
+    struct copy_error *error = impl_from_IBackgroundCopyError(iface);
+    FIXME("(%p)->(%p)\n", error, pContextDescription);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI copy_error_GetProtocol(
+    IBackgroundCopyError *iface,
+    LPWSTR *pProtocol)
+{
+    struct copy_error *error = impl_from_IBackgroundCopyError(iface);
+    FIXME("(%p)->(%p)\n", error, pProtocol);
+    return E_NOTIMPL;
+}
+
+static const IBackgroundCopyErrorVtbl copy_error_vtbl =
+{
+    copy_error_QueryInterface,
+    copy_error_AddRef,
+    copy_error_Release,
+    copy_error_GetError,
+    copy_error_GetFile,
+    copy_error_GetErrorDescription,
+    copy_error_GetErrorContextDescription,
+    copy_error_GetProtocol
+};
+
+static HRESULT create_copy_error(
+    BG_ERROR_CONTEXT context,
+    HRESULT code,
+    IBackgroundCopyFile2 *file,
+    IBackgroundCopyError **obj)
+{
+    struct copy_error *error;
+
+    TRACE("context %u code %08x file %p\n", context, code, file);
+
+    if (!(error = HeapAlloc(GetProcessHeap(), 0, sizeof(*error) ))) return E_OUTOFMEMORY;
+    error->IBackgroundCopyError_iface.lpVtbl = &copy_error_vtbl;
+    error->refs    = 1;
+    error->context = context;
+    error->code    = code;
+    error->file    = file;
+    if (error->file) IBackgroundCopyFile2_AddRef(error->file);
+
+    *obj = &error->IBackgroundCopyError_iface;
+    TRACE("returning iface %p\n", *obj);
+    return S_OK;
+}
+
 static inline BOOL is_job_done(const BackgroundCopyJobImpl *job)
 {
     return job->state == BG_JOB_STATE_CANCELLED || job->state == BG_JOB_STATE_ACKNOWLEDGED;
 }
 
-static inline BackgroundCopyJobImpl *impl_from_IBackgroundCopyJob2(IBackgroundCopyJob2 *iface)
+static inline BackgroundCopyJobImpl *impl_from_IBackgroundCopyJob3(IBackgroundCopyJob3 *iface)
 {
-    return CONTAINING_RECORD(iface, BackgroundCopyJobImpl, IBackgroundCopyJob2_iface);
+    return CONTAINING_RECORD(iface, BackgroundCopyJobImpl, IBackgroundCopyJob3_iface);
 }
 
 static HRESULT WINAPI BackgroundCopyJob_QueryInterface(
-    IBackgroundCopyJob2 *iface, REFIID riid, void **obj)
+    IBackgroundCopyJob3 *iface, REFIID riid, void **obj)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
 
     TRACE("(%p)->(%s %p)\n", This, debugstr_guid(riid), obj);
 
-    if (IsEqualGUID(riid, &IID_IUnknown)
-        || IsEqualGUID(riid, &IID_IBackgroundCopyJob)
-        || IsEqualGUID(riid, &IID_IBackgroundCopyJob2))
+    if (IsEqualGUID(riid, &IID_IUnknown) ||
+        IsEqualGUID(riid, &IID_IBackgroundCopyJob) ||
+        IsEqualGUID(riid, &IID_IBackgroundCopyJob2) ||
+        IsEqualGUID(riid, &IID_IBackgroundCopyJob3))
     {
-        *obj = iface;
-        IBackgroundCopyJob2_AddRef(iface);
-        return S_OK;
+        *obj = &This->IBackgroundCopyJob3_iface;
+    }
+    else
+    {
+        *obj = NULL;
+        return E_NOINTERFACE;
     }
 
-    *obj = NULL;
-    return E_NOINTERFACE;
+    IBackgroundCopyJob3_AddRef(iface);
+    return S_OK;
 }
 
-static ULONG WINAPI BackgroundCopyJob_AddRef(IBackgroundCopyJob2 *iface)
+static ULONG WINAPI BackgroundCopyJob_AddRef(IBackgroundCopyJob3 *iface)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
     TRACE("(%p)->(%d)\n", This, ref);
     return ref;
 }
 
-static ULONG WINAPI BackgroundCopyJob_Release(IBackgroundCopyJob2 *iface)
+static ULONG WINAPI BackgroundCopyJob_Release(IBackgroundCopyJob3 *iface)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
     TRACE("(%p)->(%d)\n", This, ref);
@@ -90,11 +254,11 @@ static ULONG WINAPI BackgroundCopyJob_Release(IBackgroundCopyJob2 *iface)
 /*** IBackgroundCopyJob methods ***/
 
 static HRESULT WINAPI BackgroundCopyJob_AddFileSet(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     ULONG cFileCount,
     BG_FILE_INFO *pFileSet)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     HRESULT hr = S_OK;
     ULONG i;
 
@@ -124,41 +288,41 @@ static HRESULT WINAPI BackgroundCopyJob_AddFileSet(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_AddFile(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPCWSTR RemoteUrl,
     LPCWSTR LocalName)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     BG_FILE_INFO file;
 
     TRACE("(%p)->(%s %s)\n", This, debugstr_w(RemoteUrl), debugstr_w(LocalName));
 
     file.RemoteName = (LPWSTR)RemoteUrl;
     file.LocalName = (LPWSTR)LocalName;
-    return IBackgroundCopyJob2_AddFileSet(iface, 1, &file);
+    return IBackgroundCopyJob3_AddFileSet(iface, 1, &file);
 }
 
 static HRESULT WINAPI BackgroundCopyJob_EnumFiles(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     IEnumBackgroundCopyFiles **enum_files)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     TRACE("(%p)->(%p)\n", This, enum_files);
     return EnumBackgroundCopyFilesConstructor(This, enum_files);
 }
 
 static HRESULT WINAPI BackgroundCopyJob_Suspend(
-    IBackgroundCopyJob2 *iface)
+    IBackgroundCopyJob3 *iface)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p): stub\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_Resume(
-    IBackgroundCopyJob2 *iface)
+    IBackgroundCopyJob3 *iface)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     HRESULT rv = S_OK;
 
     TRACE("(%p)\n", This);
@@ -184,17 +348,17 @@ static HRESULT WINAPI BackgroundCopyJob_Resume(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_Cancel(
-    IBackgroundCopyJob2 *iface)
+    IBackgroundCopyJob3 *iface)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p): stub\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_Complete(
-    IBackgroundCopyJob2 *iface)
+    IBackgroundCopyJob3 *iface)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     HRESULT rv = S_OK;
 
     TRACE("(%p)\n", This);
@@ -235,20 +399,20 @@ static HRESULT WINAPI BackgroundCopyJob_Complete(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetId(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     GUID *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     TRACE("(%p)->(%p)\n", This, pVal);
     *pVal = This->jobId;
     return S_OK;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetType(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_TYPE *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
 
     TRACE("(%p)->(%p)\n", This, pVal);
 
@@ -260,10 +424,10 @@ static HRESULT WINAPI BackgroundCopyJob_GetType(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetProgress(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_PROGRESS *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
 
     TRACE("(%p)->(%p)\n", This, pVal);
 
@@ -281,19 +445,19 @@ static HRESULT WINAPI BackgroundCopyJob_GetProgress(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetTimes(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_TIMES *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, pVal);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetState(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_STATE *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
 
     TRACE("(%p)->(%p)\n", This, pVal);
 
@@ -306,37 +470,41 @@ static HRESULT WINAPI BackgroundCopyJob_GetState(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetError(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     IBackgroundCopyError **ppError)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
-    FIXME("(%p)->(%p): stub\n", This, ppError);
-    return E_NOTIMPL;
+    BackgroundCopyJobImpl *job = impl_from_IBackgroundCopyJob3(iface);
+
+    TRACE("(%p)->(%p)\n", job, ppError);
+
+    if (!job->error.context) return BG_E_ERROR_INFORMATION_UNAVAILABLE;
+
+    return create_copy_error(job->error.context, job->error.code, job->error.file, ppError);
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetOwner(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPWSTR *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, pVal);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetDisplayName(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPCWSTR Val)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%s): stub\n", This, debugstr_w(Val));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetDisplayName(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPWSTR *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
 
     TRACE("(%p)->(%p)\n", This, pVal);
 
@@ -344,10 +512,10 @@ static HRESULT WINAPI BackgroundCopyJob_GetDisplayName(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetDescription(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPCWSTR Val)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     static const int max_description_len = 1024;
     HRESULT hr = S_OK;
     int len;
@@ -380,10 +548,10 @@ static HRESULT WINAPI BackgroundCopyJob_SetDescription(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetDescription(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPWSTR *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
 
     TRACE("(%p)->(%p)\n", This, pVal);
 
@@ -391,28 +559,28 @@ static HRESULT WINAPI BackgroundCopyJob_GetDescription(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetPriority(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_PRIORITY Val)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%d): stub\n", This, Val);
     return S_OK;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetPriority(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_PRIORITY *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, pVal);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetNotifyFlags(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     ULONG Val)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     static const ULONG valid_flags = BG_NOTIFY_JOB_TRANSFERRED |
                                      BG_NOTIFY_JOB_ERROR |
                                      BG_NOTIFY_DISABLE |
@@ -428,10 +596,10 @@ static HRESULT WINAPI BackgroundCopyJob_SetNotifyFlags(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetNotifyFlags(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     ULONG *pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
 
     TRACE("(%p)->(%p)\n", This, pVal);
 
@@ -443,10 +611,10 @@ static HRESULT WINAPI BackgroundCopyJob_GetNotifyFlags(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetNotifyInterface(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     IUnknown *Val)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     HRESULT hr = S_OK;
 
     TRACE("(%p)->(%p)\n", This, Val);
@@ -473,10 +641,10 @@ static HRESULT WINAPI BackgroundCopyJob_SetNotifyInterface(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetNotifyInterface(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     IUnknown **pVal)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
 
     TRACE("(%p)->(%p)\n", This, pVal);
 
@@ -490,7 +658,7 @@ static HRESULT WINAPI BackgroundCopyJob_GetNotifyInterface(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetMinimumRetryDelay(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     ULONG Seconds)
 {
     FIXME("%u\n", Seconds);
@@ -498,150 +666,190 @@ static HRESULT WINAPI BackgroundCopyJob_SetMinimumRetryDelay(
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetMinimumRetryDelay(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     ULONG *Seconds)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, Seconds);
     *Seconds = 30;
     return S_OK;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetNoProgressTimeout(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     ULONG Seconds)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%d): stub\n", This, Seconds);
     return S_OK;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetNoProgressTimeout(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     ULONG *Seconds)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, Seconds);
     *Seconds = 900;
     return S_OK;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetErrorCount(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     ULONG *Errors)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, Errors);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetProxySettings(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_PROXY_USAGE ProxyUsage,
     const WCHAR *ProxyList,
     const WCHAR *ProxyBypassList)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%d %s %s): stub\n", This, ProxyUsage, debugstr_w(ProxyList), debugstr_w(ProxyBypassList));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetProxySettings(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_PROXY_USAGE *pProxyUsage,
     LPWSTR *pProxyList,
     LPWSTR *pProxyBypassList)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p %p %p): stub\n", This, pProxyUsage, pProxyList, pProxyBypassList);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_TakeOwnership(
-    IBackgroundCopyJob2 *iface)
+    IBackgroundCopyJob3 *iface)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p): stub\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetNotifyCmdLine(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPCWSTR prog,
     LPCWSTR params)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%s %s): stub\n", This, debugstr_w(prog), debugstr_w(params));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetNotifyCmdLine(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPWSTR *prog,
     LPWSTR *params)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p %p): stub\n", This, prog, params);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetReplyProgress(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_JOB_REPLY_PROGRESS *progress)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, progress);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetReplyData(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     byte **pBuffer,
     UINT64 *pLength)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p %p): stub\n", This, pBuffer, pLength);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetReplyFileName(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPCWSTR filename)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%s): stub\n", This, debugstr_w(filename));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_GetReplyFileName(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     LPWSTR *pFilename)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, pFilename);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_SetCredentials(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_AUTH_CREDENTIALS *cred)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%p): stub\n", This, cred);
     return S_OK;
 }
 
 static HRESULT WINAPI BackgroundCopyJob_RemoveCredentials(
-    IBackgroundCopyJob2 *iface,
+    IBackgroundCopyJob3 *iface,
     BG_AUTH_TARGET target,
     BG_AUTH_SCHEME scheme)
 {
-    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob2(iface);
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
     FIXME("(%p)->(%d %d): stub\n", This, target, scheme);
     return S_OK;
 }
 
-static const IBackgroundCopyJob2Vtbl BackgroundCopyJobVtbl =
+static HRESULT WINAPI BackgroundCopyJob_ReplaceRemotePrefix(
+    IBackgroundCopyJob3 *iface,
+    LPCWSTR OldPrefix,
+    LPCWSTR NewPrefix)
+{
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
+    FIXME("(%p)->(%s %s): stub\n", This, debugstr_w(OldPrefix), debugstr_w(NewPrefix));
+    return S_OK;
+}
+
+static HRESULT WINAPI BackgroundCopyJob_AddFileWithRanges(
+    IBackgroundCopyJob3 *iface,
+    LPCWSTR RemoteUrl,
+    LPCWSTR LocalName,
+    DWORD RangeCount,
+    BG_FILE_RANGE Ranges[])
+{
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
+    FIXME("(%p)->(%s %s %u %p): stub\n", This, debugstr_w(RemoteUrl), debugstr_w(LocalName), RangeCount, Ranges);
+    return S_OK;
+}
+
+static HRESULT WINAPI BackgroundCopyJob_SetFileACLFlags(
+    IBackgroundCopyJob3 *iface,
+    DWORD Flags)
+{
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
+    FIXME("(%p)->(%x): stub\n", This, Flags);
+    return S_OK;
+}
+
+static HRESULT WINAPI BackgroundCopyJob_GetFileACLFlags(
+    IBackgroundCopyJob3 *iface,
+    DWORD *Flags)
+{
+    BackgroundCopyJobImpl *This = impl_from_IBackgroundCopyJob3(iface);
+    FIXME("(%p)->(%p): stub\n", This, Flags);
+    return S_OK;
+}
+
+static const IBackgroundCopyJob3Vtbl BackgroundCopyJob3Vtbl =
 {
     BackgroundCopyJob_QueryInterface,
     BackgroundCopyJob_AddRef,
@@ -685,7 +893,11 @@ static const IBackgroundCopyJob2Vtbl BackgroundCopyJobVtbl =
     BackgroundCopyJob_SetReplyFileName,
     BackgroundCopyJob_GetReplyFileName,
     BackgroundCopyJob_SetCredentials,
-    BackgroundCopyJob_RemoveCredentials
+    BackgroundCopyJob_RemoveCredentials,
+    BackgroundCopyJob_ReplaceRemotePrefix,
+    BackgroundCopyJob_AddFileWithRanges,
+    BackgroundCopyJob_SetFileACLFlags,
+    BackgroundCopyJob_GetFileACLFlags
 };
 
 HRESULT BackgroundCopyJobConstructor(LPCWSTR displayName, BG_JOB_TYPE type, GUID *job_id, BackgroundCopyJobImpl **job)
@@ -700,7 +912,7 @@ HRESULT BackgroundCopyJobConstructor(LPCWSTR displayName, BG_JOB_TYPE type, GUID
     if (!This)
         return E_OUTOFMEMORY;
 
-    This->IBackgroundCopyJob2_iface.lpVtbl = &BackgroundCopyJobVtbl;
+    This->IBackgroundCopyJob3_iface.lpVtbl = &BackgroundCopyJob3Vtbl;
     InitializeCriticalSection(&This->cs);
     This->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": BackgroundCopyJobImpl.cs");
 
@@ -741,6 +953,10 @@ HRESULT BackgroundCopyJobConstructor(LPCWSTR displayName, BG_JOB_TYPE type, GUID
     This->callback = NULL;
     This->callback2 = FALSE;
 
+    This->error.context = 0;
+    This->error.code = 0;
+    This->error.file = NULL;
+
     *job = This;
 
     TRACE("created job %s:%p\n", debugstr_guid(&This->jobId), This);
