@@ -44,7 +44,6 @@ MAKE_FUNCPTR(XcursorLibraryLoadCursor);
 #endif /* SONAME_LIBXCURSOR */
 
 #define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #define OEMRESOURCE
 #include "windef.h"
 #include "winbase.h"
@@ -131,6 +130,7 @@ static Cursor create_cursor( HANDLE handle );
 
 #ifdef HAVE_X11_EXTENSIONS_XINPUT2_H
 static BOOL xinput2_available;
+static BOOL broken_rawevents;
 #define MAKE_FUNCPTR(f) static typeof(f) * p##f
 MAKE_FUNCPTR(XIFreeDeviceInfo);
 MAKE_FUNCPTR(XIQueryDevice);
@@ -814,10 +814,10 @@ static const struct system_cursors comctl32_cursors[] =
     { 102, "move" },
     { 104, "copy" },
     { 105, "left_ptr" },
-    { 106, "row-resize" },
-    { 107, "row-resize" },
+    { 106, "col-resize" },
+    { 107, "col-resize" },
     { 108, "hand2" },
-    { 135, "col-resize" },
+    { 135, "row-resize" },
     { 0 }
 };
 
@@ -875,7 +875,7 @@ static const struct cursor_font_fallback fallbacks[] =
     { "circle",              XC_circle },
     { "clock",               XC_clock },
     { "coffee_mug",          XC_coffee_mug },
-    { "col-resize",          XC_sb_v_double_arrow },
+    { "col-resize",          XC_sb_h_double_arrow },
     { "cross",               XC_cross },
     { "cross_reverse",       XC_cross_reverse },
     { "crosshair",           XC_crosshair },
@@ -912,7 +912,7 @@ static const struct cursor_font_fallback fallbacks[] =
     { "right_side",          XC_right_side },
     { "right_tee",           XC_right_tee },
     { "rightbutton",         XC_rightbutton },
-    { "row-resize",          XC_sb_h_double_arrow },
+    { "row-resize",          XC_sb_v_double_arrow },
     { "rtl_logo",            XC_rtl_logo },
     { "sailboat",            XC_sailboat },
     { "sb_down_arrow",       XC_sb_down_arrow },
@@ -1579,7 +1579,15 @@ void X11DRV_MotionNotify( HWND hwnd, XEvent *xev )
     if (!hwnd)
     {
         struct x11drv_thread_data *thread_data = x11drv_thread_data();
-        if (thread_data->warp_serial && (long)(event->serial - thread_data->warp_serial) < 0) return;
+        if (thread_data->warp_serial)
+        {
+            if ((long)(event->serial - thread_data->warp_serial) < 0)
+            {
+                TRACE( "pos %d,%d old serial %lu, ignoring\n", input.u.mi.dx, input.u.mi.dy, event->serial );
+                return;
+            }
+            thread_data->warp_serial = 0;  /* we caught up now */
+        }
     }
 
     send_mouse_input( hwnd, event->window, event->state, &input );
@@ -1669,7 +1677,7 @@ static void X11DRV_RawMotion( XGenericEventCookie *xev )
         break;
     }
 
-    if (thread_data->warp_serial)
+    if (broken_rawevents && thread_data->warp_serial)
     {
         if ((long)(xev->serial - thread_data->warp_serial) < 0)
         {
@@ -1716,6 +1724,12 @@ void X11DRV_XInput2_Init(void)
 #undef LOAD_FUNCPTR
 
     xinput2_available = XQueryExtension( gdi_display, "XInputExtension", &xinput2_opcode, &event, &error );
+
+    /* Until version 1.10.4 rawinput was broken in XOrg, see
+     * https://bugs.freedesktop.org/show_bug.cgi?id=30068 */
+    broken_rawevents  = strstr(XServerVendor( gdi_display ), "X.Org") &&
+                        XVendorRelease( gdi_display ) < 11004000;
+
 #else
     TRACE( "X Input 2 support not compiled in.\n" );
 #endif
