@@ -206,10 +206,11 @@ void *alloc_object( const struct object_ops *ops )
     struct object *obj = mem_alloc( ops->size );
     if (obj)
     {
-        obj->refcount = 1;
-        obj->ops      = ops;
-        obj->name     = NULL;
-        obj->sd       = NULL;
+        obj->refcount     = 1;
+        obj->handle_count = 0;
+        obj->ops          = ops;
+        obj->name         = NULL;
+        obj->sd           = NULL;
         list_init( &obj->wait_queue );
 #ifdef DEBUG_OBJECTS
         list_add_head( &object_list, &obj->obj_list );
@@ -306,6 +307,7 @@ void release_object( void *ptr )
     assert( obj->refcount );
     if (!--obj->refcount)
     {
+        assert( !obj->handle_count );
         /* if the refcount is 0, nobody can be in the wait queue */
         assert( list_empty( &obj->wait_queue ));
         obj->ops->destroy( obj );
@@ -451,6 +453,7 @@ int set_sd_defaults_from_token( struct object *obj, const struct security_descri
         owner = token_get_user( token );
         new_sd.owner_len = security_sid_len( owner );
     }
+    else new_sd.owner_len = 0;
 
     if (set_info & GROUP_SECURITY_INFORMATION && sd->group_len)
     {
@@ -467,6 +470,7 @@ int set_sd_defaults_from_token( struct object *obj, const struct security_descri
         group = token_get_primary_group( token );
         new_sd.group_len = security_sid_len( group );
     }
+    else new_sd.group_len = 0;
 
     new_sd.control |= SE_SACL_PRESENT;
     sacl = sd_get_sacl( sd, &present );
@@ -479,9 +483,7 @@ int set_sd_defaults_from_token( struct object *obj, const struct security_descri
         if (obj->sd && present)
             new_sd.sacl_len = obj->sd->sacl_len;
         else
-        {
             new_sd.sacl_len = 0;
-        }
     }
 
     new_sd.control |= SE_DACL_PRESENT;
@@ -499,6 +501,7 @@ int set_sd_defaults_from_token( struct object *obj, const struct security_descri
             dacl = token_get_default_dacl( token );
             new_sd.dacl_len = dacl->AclSize;
         }
+        else new_sd.dacl_len = 0;
     }
 
     ptr = mem_alloc( sizeof(new_sd) + new_sd.owner_len + new_sd.group_len +
